/*
 * @(#)79    1.1  test/src/HTTPSPanel.java, jsse.test, jsse1  6/2/00  16:07:00
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * IBM Security Software Development Kit, Java (tm) Technology Edition
 *
 * (C) Copyright IBM Corp. 2000 All Rights Reserved.
 *
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * ===========================================================================
 */
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.ImageIcon;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.File;
import java.net.URL;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.security.Security;
import java.util.Properties;  

/*
 * HTTPSPanel
 *
 * Sample application to demonstrate accessing various web sites and view the HTML returned from 
 * those sites using the defaulted or specified trustStore.  The java protocol handler of the
 * com.ibm.net.ssl.internal.www.protocol class is used.
 *
 * You can build this sample program class by issuing the following command:
 *
 *     javac HTTPSPanel.java
 *
 * Make sure that the ibmjsse.jar file is in your CLASSPATH or extensions directory.
 *
 * To run this sample program, execute the following command:
 *     
 *     java HTTPSPanel 
 *
 * Usage Instructions:
 *
 * Manual Proxy Configuration:
 *   Check box to enable manual proxy configuration
 *     Fill in the Socks Server and Sock Server Port 
 *   Uncheck box if no socks server is desired.
 *
 * Use your own Certificate Database as trustStore
 *   If box is unchecked, then the system default trustStore is used.
 *   Check box to enable you to specify the Certificate Database to be used
 *      Select open to specify where to find the requested Certificate Database
 *      Two Certificate Databases are provided:  1) cacerts and 2) testkeys
 *      IKEYMAN can be used to create your own Certificate Database.
 *
 * Netsite
 *    Provide a complete URL or select one of the web site addresses provided
 *
 * Connect and Exit
 *    Press Connect to display the HTML for the web site selected.
 *    Exit terminates the application.
 *
 */

public class HTTPSPanel extends JFrame
{
	public HTTPSPanel()
	{
	    /*
	     *  Set up HTTPS Panel
	     */
	     
		setTitle("HTTPS");
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		getContentPane().setLayout(new BorderLayout(0,0));
		setSize(490,750);
		setVisible(false);
		JPanel1.setLayout(null);
		getContentPane().add("Center", JPanel1);
		JPanel1.setBounds(0,0,490,750);
		checkbox.setText("Manual Proxy Configuration");
		checkbox.setActionCommand("Manual Proxy Configuration");
		checkbox.setSelected(true);
		JPanel1.add(checkbox);
		checkbox.setBounds(24,56,276,24);
		socksname.setText("Socks:");
		JPanel1.add(socksname);
		socksname.setBounds(36,84,48,20);
		socks.setText("socks.austin.ibm.com");
		JPanel1.add(socks);
		socks.setBounds(84,84,228,24);
		portname.setText("Port:");
		JPanel1.add(portname);
		portname.setBounds(348,80,49,24);
		port.setText("1080");
		JPanel1.add(port);
		port.setBounds(384,80,75,24);
		JLabel1.setText("IBM JSSE1.0 HTTPS Test Panel");
		JPanel1.add(JLabel1);
		JLabel1.setFont(new Font("Dialog", Font.BOLD, 18));
		JLabel1.setBounds(84,8,300,41);
		urlname.setText("Netsite:");
		JPanel1.add(urlname);
		urlname.setBounds(24,216,48,18);
		JComboBox1.setEditable(true);
        
        // Put sample web addresses in drop down box
        JComboBox1.insertItemAt("https://ezonline.ibmtefcu.org/", 0);
        JComboBox1.insertItemAt("https://mil401k.metlife.com/ibm/", 1);
        JComboBox1.insertItemAt("https://webbanking.waterhousebank.com/", 2);
		JPanel1.add(JComboBox1);
		JComboBox1.setBackground(new java.awt.Color(204,204,204));
		JComboBox1.setBounds(84,216,372,24);
		JButton1.setText("Connect...");
		JButton1.setActionCommand("Connect...");
		JPanel1.add(JButton1);
		JButton1.setBounds(24,264,108,24);
		JPanel1.add(textArea1);
		textArea1.setBounds(24,300,432,420);
		JButton2.setText("Exit");
		JButton2.setActionCommand("Exit");
		JPanel1.add(JButton2);
		JButton2.setBounds(360,264,98,24);
		cert.setText("Use your own Certificate Database as trustStore");
		cert.setActionCommand("Use your own Certificate Database as trustStore");
		JPanel1.add(cert);
		cert.setBounds(24,120,300,36);
		JTextField1.setText(" ");
		JPanel1.add(JTextField1);
		JTextField1.setBounds(144,156,312,23);
		JButton3.setText("Open");
		JButton3.setActionCommand("Open");
        JButton3.setEnabled(false);
		JPanel1.add(JButton3);
		JButton3.setFont(new Font("Dialog", Font.BOLD, 12));
		JButton3.setBounds(36,156,72,24);
		openFileDialog.setMode(FileDialog.LOAD);
		SymWindow aSymWindow = new SymWindow();
		this.addWindowListener(aSymWindow);
		SymAction lSymAction = new SymAction();
		JButton2.addActionListener(lSymAction);
		JButton1.addActionListener(lSymAction);
		checkbox.addActionListener(lSymAction);
		JButton3.addActionListener(lSymAction);
		cert.addActionListener(lSymAction);
        openFileDialog.setMode(FileDialog.LOAD);
		openFileDialog.setTitle("Open");
	}

	public HTTPSPanel(String sTitle)
	{
		this();
		setTitle(sTitle);
	}
	
	/**
	 * The entry point for this application.
	 * Sets the Look and Feel to the System Look and Feel.
	 * Creates a new HTTPSPanel and makes it visible.
	 */
	static public void main(String args[])
	{
            // Check if provider IBM Jsse Provider is installed.
            try {
                if (Security.getProvider("com.ibm.jsse.IBMJSSEProvider") == null) {
                    // IBMJSSE Provider is not installed, install it
                    Security.addProvider(new com.ibm.jsse.IBMJSSEProvider());
                }
            }catch (Exception e) {
                System.out.println("Cannot install provider: " + e.getMessage());
                System.exit(1);
            }

	    try {
            
	       	HTTPSPanel frame = new HTTPSPanel();
                Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	        frame.setLocation(screenSize.width/2 - 510/2,
			  screenSize.height/2 - 700/2);
                frame.setVisible(true);
	     } catch (Throwable t) {
	         t.printStackTrace();
		 System.exit(1);
	     }
	}

    /**
     * Notifies this component that it has been added to a container
     * This method should be called by <code>Container.add</code>, and 
     * not by user code directly.
     * Overridden here to adjust the size of the frame if needed.
     * @see java.awt.Container#removeNotify
     */
	public void addNotify()
	{
		// Record the size of the window prior to calling parents addNotify.
		Dimension size = getSize();
		
		super.addNotify();
		
		if (frameSizeAdjusted)
			return;
		frameSizeAdjusted = true;
		
		// Adjust size of frame according to the insets and menu bar
         JMenuBar menuBar = getRootPane().getJMenuBar();
		int menuBarHeight = 0;
		if (menuBar != null)
		    menuBarHeight = menuBar.getPreferredSize().height;
		Insets insets = getInsets();
		setSize(insets.left + insets.right + size.width, insets.top + insets.bottom + size.height + menuBarHeight);
	}

	// Used by addNotify
	boolean frameSizeAdjusted = false;

    JPanel JPanel1 = new JPanel();
	JCheckBox checkbox = new JCheckBox();
	JLabel socksname = new JLabel();
	JTextField socks = new JTextField();
	JLabel portname = new JLabel();
	JTextField port = new JTextField();
	JLabel JLabel1 = new JLabel();
	JLabel urlname = new JLabel();
	JComboBox JComboBox1 = new JComboBox();
	JButton JButton1 = new JButton();
	java.awt.TextArea textArea1 = new java.awt.TextArea("",5,40,TextArea.SCROLLBARS_VERTICAL_ONLY);
	JButton JButton2 = new JButton();
	JCheckBox cert = new JCheckBox();
	JTextField JTextField1 = new JTextField();
	JButton JButton3 = new JButton();
	java.awt.FileDialog openFileDialog = new java.awt.FileDialog(this);

    // User pressed the exit button
	void exitApplication()
	{
		try {
	    	// Beep
	    	Toolkit.getDefaultToolkit().beep();
	    	// Show a confirmation dialog
	    	int reply = JOptionPane.showConfirmDialog(this, 
	    	                                          "Do you really want to exit?", 
	    	                                          "Exit" , 
	    	                                          JOptionPane.YES_NO_OPTION, 
	    	                                          JOptionPane.QUESTION_MESSAGE);
			// If the confirmation was affirmative, handle exiting.
			if (reply == JOptionPane.YES_OPTION)
			{
		    	this.setVisible(false);    // hide the Frame
		    	this.dispose();            // free the system resources
		    	System.exit(0);            // close the application
			}
		} catch (Exception e) {
		}
	}

	class SymWindow extends java.awt.event.WindowAdapter
	{
		public void windowClosing(java.awt.event.WindowEvent event)
		{
			Object object = event.getSource();
			if (object == HTTPSPanel.this)
				HTTPSPanel_windowClosing(event);
		}
	}

	void HTTPSPanel_windowClosing(java.awt.event.WindowEvent event)
	{
	}

    // Listener
	class SymAction implements java.awt.event.ActionListener
	{
		public void actionPerformed(java.awt.event.ActionEvent event)
		{
			Object object = event.getSource();
			if (object == JButton2)
				JButton2_actionPerformed(event);
			else if (object == JButton1)
				JButton1_actionPerformed(event);
			else if (object == checkbox)
				checkbox_actionPerformed(event);
			else if (object == JButton3)
				JButton3_actionPerformed(event);
			else if (object == cert)
				cert_actionPerformed(event);
			
			
			
		}
	}

    // Exit button pressed
	void JButton2_actionPerformed(java.awt.event.ActionEvent event)
	{
        exitApplication();
	}


    // Connect... button pressed.  Process.
	void JButton1_actionPerformed(java.awt.event.ActionEvent event)
	{

		String socksServer = socks.getText();
		String socksPort = port.getText();
		String url = (String)JComboBox1.getSelectedItem();
		System.out.println("selected url is " + url);
		textArea1.requestFocus();
		textArea1.append("Connecting to " + url + "...\n");
		Properties properties = System.getProperties();
        properties.put("java.protocol.handler.pkgs", "com.ibm.net.ssl.internal.www.protocol");
        if (checkbox.isSelected()) {
            if (!socksServer.equals("")) 
                properties.put("socksProxyHost", socksServer);
            if (!socksPort.equals("")) 
                properties.put("socksProxyPort", socksPort);
        }
        else {
            properties.remove("socksProxyHost");
            properties.remove("socksProxyPort");
        }
        if (cert.isSelected()) {
            String trustStore = JTextField1.getText();
            if (trustStore != null){
                properties.put("javax.net.ssl.trustStore", trustStore);
            }
            System.out.println("use your own trust store " + trustStore);
        }
        else {
            properties.remove("javax.net.ssl.trustStore");
        }
        System.setProperties(properties);
        try {
    		URL page = new URL(url); 
	    	HttpURLConnection urlc = (HttpURLConnection)page.openConnection();
		    urlc.setUseCaches(false); // Don't look at possibly cached data
		    textArea1.append("Waiting for reply ... \n");
		    textArea1.append("Response code = " + urlc.getResponseCode() + "\n");
            textArea1.append("Response message = " + urlc.getResponseMessage() + "\n");
            textArea1.append("Content-type = " + urlc.getContentType()+ "\n"); // See what's here
		    textArea1.append("Content-length = " + urlc.getContentLength()+ "\n"); // See how much of it there is
		    // Read it all and print it out
    		BufferedReader br = new BufferedReader(new InputStreamReader(urlc.getInputStream()));
	    	String buffer = "\n";
		    while (buffer != null) {
		        try {
		            textArea1.append(buffer);
		            buffer = br.readLine();
		        }
		        catch (IOException ioe) {
		            ioe.printStackTrace();
		            break;
		        }
		    }
		}
		catch (MalformedURLException mue) {
			textArea1.append(url + " is not a URL that can be resolved\n");
		}
		catch (IOException ie) {
			ie.printStackTrace();
		}

	}

    // Handle Manual Proxy Configuration Checkbox
	void checkbox_actionPerformed(java.awt.event.ActionEvent event)
	{
	    if (!(checkbox.isSelected()))	{
	       port.setEnabled(false);
	       socks.setEnabled(false);
	    }
	    else {
	        port.setEnabled(true);
	        socks.setEnabled(true);
	    }
			 
	}

    // Handle Selecting the requested Certificate Database as trustStore 
	void JButton3_actionPerformed(java.awt.event.ActionEvent event)
	{
	try {
			// openFileDialog Show the FileDialog
			openFileDialog.setVisible(true);
            String directory = openFileDialog.getDirectory();
            String file = openFileDialog.getFile();
            if ((directory != null) && (file != null)) {
                JTextField1.setText(directory + File.separator + file);
            }
		} catch (Exception e) {
		}

			 
	}

    // Enable Use your own Certificate Database as trustStore open button
	void cert_actionPerformed(java.awt.event.ActionEvent event)
	{
		if (cert.isSelected()) {
		    JButton3.setEnabled(true);
		}
		else
		    JButton3.setEnabled(false);
	}
}
