/*
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * IBM Security Software Development Kit, Java (tm) Technology Edition
 *
 * (C) Copyright IBM Corp. 2000 All Rights Reserved.
 *
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * ===========================================================================
 */

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.File;
import java.net.URL;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.security.Security;
import java.util.Properties;  
import com.ibm.net.ssl.HttpsURLConnection;


/*
 * HTTPTest
 *
 * Sample program to demonstrate accessing an SSL or non-SSL webserver using the java protocol handler of
 * the com.ibm.net.ssl.internal.www.protocol class.  The URL is specified with the http: or https: prefix.
 * The HTML returned from this site is displayed.
 * 
 * You can build this sample program class by issuing the following command:
 *
 *     javac HTTPTest.java
 *
 * Make sure that the ibmjsse.jar file is in your CLASSPATH or extensions directory.
 *
 * To run this sample program, execute the following command:
 *     
 *     java HTTPTest %URL%
 *
 * where %URL% is the URL of the webserver with the http: or https: prefix. It is a required parameter.
 * 
 */

public class HTTPTest {
    public static void main(String[] args) {
        String url = null;
        try {
            if ((args.length == 0) || (args == null)) {
                System.out.println("Provider a URL input with https:// or http:// as prefix at command line.");
                System.out.println("Some available https URL are:\n");
                System.out.println("\t https://ezonline.ibmtefcu.org/");
                System.out.println("\t https://mil401k.metlife.com/ibm/");
                System.out.println("\t https://webbanking.waterhousebank.com/");
                System.exit(0);
            }
            
            // Check if provider IBMJSSE Provider is installed.
            try {
                if (Security.getProvider("com.ibm.jsse.IBMJSSEProvider") == null) {
                    // IBMJSSE Provider is not installed, install it
                    Security.addProvider(new com.ibm.jsse.IBMJSSEProvider());
                }
            }catch (Exception e) {
                System.out.println ("Cannot install provider: " + e.getMessage());
                System.exit(1);
            }

            /*
             *  Add com.ibm.net.ssl.internal.www.protocol as the java protocol handler.
             */
            
            Properties properties = System.getProperties();
            properties.put("java.protocol.handler.pkgs", "com.ibm.net.ssl.internal.www.protocol");
            
            url = args[0];
    		URL page = new URL(url); 
    		
    		// get HttpURLConnection object that represents a connection to the remote object referred
    		// to by the URL.
            
            HttpURLConnection urlc = (HttpURLConnection)page.openConnection();          
		    urlc.setUseCaches(false); // Don't look at possibly cached data
		    System.out.println("Waiting for reply ...... \n");
		    System.out.println("Content-type = " + urlc.getContentType()+ "\n"); // See what's here
		    System.out.println("Content-length = " + urlc.getContentLength()+ "\n"); // See how much of it there is
            System.out.println("Response-code = " + urlc.getResponseCode() + "\n"); // Get HTTP response
                                                                                    // status
            System.out.println("Response-message = " + urlc.getResponseMessage() + "\n");  //Get the HTTP
                                                                                           // response msg
		    // Read it all and print it out
    		BufferedReader br = new BufferedReader(new InputStreamReader(urlc.getInputStream(),"8859_1"));
	    	String buffer = "\n";
		    while (buffer != null) {
		        try {
		            System.out.println(buffer);
		            buffer = br.readLine();
		        }
		        catch (IOException ioe) {
		            System.err.println(ioe.getMessage());
		            break;
		        }
		    }
		}
		catch (MalformedURLException mue) {
			System.err.println(url + " is not a URL that can be resolved\n");
		}
        
		catch (IOException ie) {
			System.err.println("IOException: " + ie.getMessage());
		}
    }
}
