/*
 * %Z%%M%    %I%  %W%  %G%  %U%
 * ===========================================================================
 * IBM Confidential
 * OCO Source Materials
 * Licensed Materials - Property of IBM
 * IBM Security Software Development Kit, Java (tm) Technology Edition
 *
 * (C) Copyright IBM Corp. 1999, 2000 All Rights Reserved.
 *
 * The source code for this program is not published or otherwise divested of
 * its trade secrets, irrespective of what has been deposited with the U.S.
 * Copyright Office.
 * ===========================================================================
 */
 
//
//     SAMPLE IBMJCE CODE
//

/*
 * Usage: This sample code demonstrate how to encrypt a message
 *        using Mars Cipher.
 *
 
 * Requirement: Usage of this sample file requires the installation of
 *        the IBMJCE provider on the running machine and the CLASSPATH
 *        points to the classes.
 *
 *
 * Running the program by typing the following in a command line:
 *
 *       java SampleMars
 */
 
import java.io.*;
import javax.crypto.*;
import java.security.PrivateKey;
import java.security.Provider;
import java.security.Security;
import java.security.PublicKey;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.Signature;
import java.security.AlgorithmParameters;
import java.security.*;
import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.spec.*;
import com.ibm.crypto.provider.MarsKeySpec;

public class SampleMars {
    static final byte[] originalMessage = "testmessage_a very long test message".getBytes();

    public static void main(String argv[]) {

        Provider p = Security.getProvider("IBMJCE");
        if (p != null) {
        System.out.println("\nIBMJCE provider name is " + p.getName());
        System.out.println("IBMJCE provider version # is " + p.getVersion());
        System.out.println("IBMJCE provider info is " + p.getInfo());
        }
        else {
            System.out.println("IBMJCE not found.");
        }
        
        try {
        Cipher c = Cipher.getInstance("Mars", "IBMJCE");
        System.out.println("My cipher algorithm name is " + c.getAlgorithm());
        } catch (Exception e) {
           System.out.println("Unexpected exception00: " + e.getMessage());
        }
 
        KeyGenerator keygen = null;
        SecretKey marskey = null;
        SecretKey marsanotherkey = null;
        SecretKey mars3key = null;
        MarsKeySpec marskeyspec = null;
        SecretKeyFactory marskeyfactory = null;
        // Testing Mars
        try {
            byte[] keybyte = "14fdjwkfcjwefknj".getBytes();
            
            
            marskeyspec = new MarsKeySpec(keybyte);
            
            marskeyfactory = SecretKeyFactory.getInstance("Mars", "IBMJCE");
            
            mars3key = marskeyfactory.generateSecret(marskeyspec);
            
            keygen = KeyGenerator.getInstance("Mars", "IBMJCE");
            
            marskey = keygen.generateKey();
            
            marsanotherkey = keygen.generateKey();
            
            final byte[] originalMsg8 = "ThisnoteisOctets".getBytes();
            
            final byte[] originalMsg  = "Message is not octet...".getBytes();
            
            byte [] myIv="1212121212121212".getBytes();
            byte [] myIv2 = "1234123412341234".getBytes();
            System.out.println("testMars::myIV length="+myIv.length);
            
            IvParameterSpec marsparamspec = new IvParameterSpec(myIv);
            
            IvParameterSpec marsparamspec2 = new IvParameterSpec(myIv2);
            
            byte[] ivbytes = marsparamspec.getIV();
            
            AlgorithmParameters marsparam = null;
            
            AlgorithmParameters marsparam2 = null;
            try {
                 marsparam = AlgorithmParameters.getInstance("Mars");
                 marsparam2 = AlgorithmParameters.getInstance("Mars");
            }catch (Exception e)
            {
                System.out.println("testMars::Exception in getting algorithm parameter get instance="+e);
            }
            
            marsparam.init(marsparamspec);
            marsparam2.init(marsparamspec2);
            Cipher myCipher;
            
            System.out.println("0.**************************************");
            myCipher = Cipher.getInstance("Mars/ECB/NoPadding", "IBMJCE");
            try {
                myCipher.init(Cipher.ENCRYPT_MODE, marsanotherkey);
                byte [] myText = "0000000000000000".getBytes();
                byte [] ciphertext = null;
            
                try {
                ciphertext = myCipher.doFinal(originalMsg8);
                }catch (Exception e) {
                    System.out.println("get ciphertext from plain EXCEPTION "+e);   
                }
                
                myCipher.init(Cipher.DECRYPT_MODE, marsanotherkey);
                
                byte []ciphertext1 = myCipher.doFinal(ciphertext);
            
                System.out.print("*   Results: ");
                
                compare(originalMsg8, ciphertext1);
                
            }catch (Exception e) {
                if (e instanceof IllegalBlockSizeException ){
                    System.out.println("Exception received!! Message size should be multiple of 8 byte.");
                }else {
                    System.out.println(e);
                }  
            }
            System.out.println("*                                      *");
            System.out.println();
            
            
            //=================================================================================
            // Mode     = ECB
            // Padding  = NoPadding
            // Param    = null
            //=================================================================================
            
            System.out.println("1.**************************************");
            System.out.println("*   Mars/ECB/NoPadding/key/msg8/NoParam*");
            System.out.println("*    uses the key spec generated key   *");
            System.out.println("*       Expected result = SAME         *");
            System.out.println("*--------------------------------------*");
            myCipher = Cipher.getInstance("Mars/ECB/NoPadding", "IBMJCE");
           
            try{
                
                myCipher.init(Cipher.ENCRYPT_MODE, mars3key);
                
                byte []ciphertext=null;
                
                ciphertext= myCipher.doFinal(originalMsg8);
                
                myCipher.init(Cipher.DECRYPT_MODE, mars3key);
                
                byte []ciphertext1 = myCipher.doFinal(ciphertext);
            
                System.out.print("*   Results: ");
                
                int z=-1;
                if (originalMsg8.length != ciphertext1.length){
                    System.out.println(" DIFFERENT LENGHT         *");
                }
                else {
                    for ( z=originalMsg8.length-1; z > 0 ; z--){
                        if (originalMsg8[z] != ciphertext1[z]) {
                            System.out.println(" DIFFERENT                *");
                            break;
                        }
                    }
                }
                if (z == 0)
                    System.out.println(" SAME                     *");
                
            }catch (Exception e) {
                if (e instanceof IllegalBlockSizeException ){
                    System.out.println("Exception received!! Message size should be multiple of 8 byte.");
                }else {
                    System.out.println(e);
                }  
            }
            System.out.println("*                                      *");
            System.out.println();
            
            //=================================================================================
            // Mode     = ECB
            // Padding  = NoPadding
            // Param    = null
            //=================================================================================
            
            System.out.println("2.**************************************");
            System.out.println("*   Mars/ECB/NoPadding/key/msg/NoParam *");
            System.out.println("*       Expected result Exception: text not multiple of 16 bytes*");
            System.out.println("*--------------------------------------*");
            myCipher = Cipher.getInstance("Mars/ECB/NoPadding", "IBMJCE");
           
            try{
                
                myCipher.init(Cipher.ENCRYPT_MODE, marskey);
            
                byte []ciphertext = myCipher.doFinal(originalMsg);
            
                myCipher.init(Cipher.DECRYPT_MODE, marskey);
            
                byte []ciphertext1 = myCipher.doFinal(ciphertext);
            
                System.out.print("*   Results: ");
                
                int z=-1;
                if (originalMsg.length != ciphertext1.length){
                    System.out.println(" DIFFERENT LENGHT         *");
                }
                else {
                    for ( z=originalMsg.length-1; z > 0 ; z--){
                        if (originalMsg[z] != ciphertext1[z]) {
                            System.out.println(" DIFFERENT                *");
                            break;
                        }
                    }
                }
                if (z == 0)
                    System.out.println(" SAME                     *");
                
            }catch (Exception e) {
                if (e instanceof IllegalBlockSizeException ){
                    System.out.println("Exception received!! Message size should be multiple of 16 byte.");
                }else {
                    System.out.println(e);
                }  
            }
            System.out.println("*                                      *");
            System.out.println();
            
            //=================================================================================
            // Mode     = ECB
            // Padding  = PKCS5Padding
            // Param    = null
            //=================================================================================
            
            System.out.println("4.**************************************");
            System.out.println("*   Mars/CBC/NoPadding/key/msg8/NoParam*");
            System.out.println("*       Expected result :Exception: Parameters missing*");
            System.out.println("*--------------------------------------*");
            myCipher = Cipher.getInstance("Mars/CBC/NoPadding", "IBMJCE");
           
            try{
                
                myCipher.init(Cipher.ENCRYPT_MODE, marskey);
            
                byte []ciphertext = myCipher.doFinal(originalMsg8);
            
                myCipher.init(Cipher.DECRYPT_MODE, marskey);
            
                byte []ciphertext1 = myCipher.doFinal(ciphertext);
            
                System.out.print("*   Results: ");
                
                int z=-1;
                if (originalMsg8.length != ciphertext1.length){
                    System.out.println(" DIFFERENT LENGHT         *");
                }
                else {
                    for ( z=originalMsg8.length-1; z > 0 ; z--){
                        if (originalMsg8[z] != ciphertext1[z]) {
                            System.out.println(" DIFFERENT                *");
                            break;
                        }
                    }
                }
                if (z == 0)
                    System.out.println(" SAME                     *");
                
            }catch (Exception e) {
                if (e instanceof IllegalBlockSizeException ){
                    System.out.println("Exception received!! Message size should be multiple of 8 byte.");
                }else {
                    System.out.println(e);
                }  
            }
            System.out.println("*                                      *");
            System.out.println();
            
            //=================================================================================
            // Mode     = CBC
            // Padding  = PKCS5Padding
            // Param    = !null
            //=================================================================================
            
            System.out.println("5.**************************************");
            System.out.println("*   Mars/CBC/NoPadding/key/msg8/Param  *");
            System.out.println("*       Expected result = DIFFERNET    *");
            System.out.println("*--------------------------------------*");
            myCipher = Cipher.getInstance("Mars/CBC/NoPadding", "IBMJCE");
           
            try{
                
                try {
                myCipher.init(Cipher.ENCRYPT_MODE, marskey, marsparam);
                }catch (Exception e)
                {
                    System.out.println("cipher init1"+e);   
                }
                
                byte []ciphertext = myCipher.doFinal(originalMsg8);
                
                
                
                try {
                myCipher.init(Cipher.DECRYPT_MODE, marskey, marsparam2);
                }catch (Exception e)
                {
                    System.out.println("cipher init1"+e);   
                }
                
                byte []ciphertext1 = myCipher.doFinal(ciphertext);
            
                System.out.print("*   Results: ");
                
                int z=-1;
                if (originalMsg8.length != ciphertext1.length){
                    System.out.println(" DIFFERENT LENGHT         *");
                }
                else {
                    for ( z=originalMsg8.length-1; z > 0 ; z--){
                        if (originalMsg8[z] != ciphertext1[z]) {
                            System.out.println(" DIFFERENT                *");
                            break;
                        }
                    }
                }
                if (z == 0)
                    System.out.println(" SAME                     *");
                
            }catch (Exception e) {
                if (e instanceof IllegalBlockSizeException ){
                    System.out.println("Exception received!! Message size should be multiple of 8 byte.");
                }else {
                    System.out.println(e);
                }  
            }
            System.out.println("*                                      *");
            System.out.println();
           
           
            //=================================================================================
            // Mode     = ECB
            // Padding  = PKCS5Padding
            // Param    = !null
            //=================================================================================
            
            System.out.println("5.**************************************");
            System.out.println("*   Mars/CBC/NoPadding/key/msg8/Param *");
            System.out.println("*       Expected result = SAME         *");
            System.out.println("*--------------------------------------*");
            myCipher = Cipher.getInstance("Mars/CBC/NoPadding", "IBMJCE");
           
            try{
                
                try {
                myCipher.init(Cipher.ENCRYPT_MODE, marskey, marsparam);
                }catch (Exception e)
                {
                    System.out.println("cipher init1"+e);   
                }
                
                byte []ciphertext = myCipher.doFinal(originalMsg8);
                
                
                
                try {
                myCipher.init(Cipher.DECRYPT_MODE, marskey, marsparam);
                }catch (Exception e)
                {
                    System.out.println("cipher init1"+e);   
                }
                
                byte []ciphertext1 = myCipher.doFinal(ciphertext);
            
                System.out.print("*   Results: ");
                
                int z=-1;
                if (originalMsg8.length != ciphertext1.length){
                    System.out.println(" DIFFERENT LENGHT         *");
                }
                else {
                    for ( z=originalMsg8.length-1; z > 0 ; z--){
                        if (originalMsg8[z] != ciphertext1[z]) {
                            System.out.println(" DIFFERENT                *");
                            break;
                        }
                    }
                }
                if (z == 0)
                    System.out.println(" SAME                     *");
                
            }catch (Exception e) {
                if (e instanceof IllegalBlockSizeException ){
                    System.out.println("Exception received!! Message size should be multiple of 8 byte.");
                }else {
                    System.out.println(e);
                }  
            }
            System.out.println("*                                      *");
            System.out.println();
           
           
            //=================================================================================
            // Mode     = ECB
            // Padding  = PKCS5Padding
            // Param    = null
            //=================================================================================
            
            System.out.println("6.**************************************");
            System.out.println("*   Mars/CBC/NoPadding/key/msg/Param*");
            System.out.println("*       Expected result :Exception: input should be multiple of 16 bytes");
            System.out.println("*--------------------------------------*");
            myCipher = Cipher.getInstance("Mars/CBC/NoPadding", "IBMJCE");
           
            try{
                
                myCipher.init(Cipher.ENCRYPT_MODE, marskey);
            
                byte []ciphertext = myCipher.doFinal(originalMsg);
            
                myCipher.init(Cipher.DECRYPT_MODE, marskey);
            
                byte []ciphertext1 = myCipher.doFinal(ciphertext);
            
                System.out.print("*   Results: ");
                
                int z=-1;
                if (originalMsg.length != ciphertext1.length){
                    System.out.println(" DIFFERENT LENGHT         *");
                }
                else {
                    for ( z=originalMsg.length-1; z > 0 ; z--){
                        if (originalMsg[z] != ciphertext1[z]) {
                            System.out.println(" DIFFERENT                *");
                            break;
                        }
                    }
                }
                if (z == 0)
                    System.out.println(" SAME                     *");
                
            }catch (Exception e) {
                if (e instanceof IllegalBlockSizeException ){
                    System.out.println("Exception received!! Message size should be multiple of 16 byte.");
                }else {
                    System.out.println(e);
                }  
            }
            System.out.println("*                                      *");
            System.out.println();
           
            //=================================================================================
            // Mode     = CBC
            // Padding  = NoPadding
            // Param    = null
            //=================================================================================
            
            System.out.println("8.**************************************");
            System.out.println("*   Mars/PCBC/NoPadding/key/msg/NoParam*");
            System.out.println("*       Result should be :Exception - not multiple of 8 bytes.");
            System.out.println("*--------------------------------------*");
           
            try{
                myCipher = Cipher.getInstance("Mars/PCBC/NoPadding", "IBMJCE");
           
                myCipher.init(Cipher.ENCRYPT_MODE, marskey);
            
                byte []ciphertext = myCipher.doFinal(originalMsg);
            
                myCipher.init(Cipher.DECRYPT_MODE, marskey);
            
                byte []ciphertext1 = myCipher.doFinal(ciphertext);
            
                System.out.print("*   Results: ");
                
                int z=-1;
                if (originalMsg.length != ciphertext1.length){
                    System.out.println(" DIFFERENT LENGHT         *");
                }
                else {
                    for ( z = originalMsg.length-1; z > 0 ; z--){
                        if (originalMsg[z] != ciphertext1[z]) {
                            System.out.println(" DIFFERENT                *");
                            break;
                        }
                    }
                }
                if (z == 0)
                    System.out.println(" SAME                     *");
                
            }catch (Exception e) {
                if (e instanceof IllegalBlockSizeException ){
                    System.out.println("Exception received!! Message size should be multiple of 8 byte.");
                }else {
                    System.out.println(e);
                }  
            }
            System.out.println("*                                      *");
            System.out.println();
           
            
        } catch (Exception ex) {
            System.out.println("testMars::Unexpected exception8: "+ex.getMessage());   
        }
        
	}
	
	public static void compare(byte[] text1, byte [] text2) 
	{
	    if (text1.length != text2.length) {
	        System.out.println("                  DIFFERENT LENGTH");   
	        
	    }
	    else {
	        int z = 0;
	        for (z = 0 ; z < text1.length; z++) {
	            if (text1[z] != text2[z]) {
	                System.out.println("            DIFFERENT");
	                break;
	            }
	                
	        }
	        if (z >= text1.length) {
	            System.out.println("            SAME");
	        }
	        
	    }

	}
	
	public static void printText(byte[] text){
	    
	    for (int z = 0; z < text.length; z++){
	        System.out.println("testMars::"+text[z]);
	    }
	}
}


