/*
 * %Z%%M%    %I%  %W%  %G%  %U%
 * ===========================================================================
 * IBM Confidential
 * OCO Source Materials
 * Licensed Materials - Property of IBM
 * IBM Security Software Development Kit, Java (tm) Technology Edition
 *
 * (C) Copyright IBM Corp. 1999, 2000 All Rights Reserved.
 *
 * The source code for this program is not published or otherwise divested of
 * its trade secrets, irrespective of what has been deposited with the U.S.
 * Copyright Office.
 * ===========================================================================
 */
 
//
// SAMPLE IBMJCE CODE
//

/*
 * Usage: This sample code demonstrates how to generate a Mars 
 *        secret key, and how to do Mars encryption and decryption.
 *
 
 * Requirement: Usage of this sample file requires the installation of
 *        the IBMJCE provider on the running machine and the CLASSPATH
 *        points to the classes.
 *
 *
 * Running the program by typing the following in a command line:
 *
 *       java SampleMarsCrypto
 */
 

import java.io.*;
import java.util.Arrays;
import java.security.Security;
import java.security.Provider;
import java.security.AlgorithmParameters;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.SecretKey;
import javax.crypto.NoSuchPaddingException;
import com.ibm.crypto.provider.*;

public class SampleMarsCrypto {
    // Plain text must be multiple of 16 bytes since Mars doesn't support any padding.
    static final byte[] plainText = "A Secret Message".getBytes();

    public static void main(String argv[]) {
        KeyGenerator MarsKeyGen;
        SecretKey MarsKey;
        Cipher cp = null;
        AlgorithmParameters params = null;
        byte[] cipherText = null, newPlainText = null;

        // Check if provider IBMJCE is installed.

        if (Security.getProvider("IBMJCE") == null) {
            // IBMJCE is not installed, install it.
            try {
                Security.addProvider
                ((Provider)Class.forName("IBMJCE").newInstance());
            }
            catch (Exception ex) {
                System.out.println("Cannot install provider: " + ex.getMessage());
                return;
            }
        }

        // Generate a Mars secret key
        try {
            MarsKeyGen = KeyGenerator.getInstance("Mars", "IBMJCE");
        } catch (NoSuchAlgorithmException ex) {
            System.out.println("Mars Key Generator not found in IBMJCE provider.");
            return;
        } catch (NoSuchProviderException ex) {
            // Should not happen.
            System.out.println("Provider IBMJCE not found.");
            return;
        }

        // Use key size = 128 bits
        MarsKeyGen.init(128);
        MarsKey = MarsKeyGen.generateKey();
        if (MarsKey == null) {
            System.out.println("Key generation failure. Secret key was not generated.");
            return;
        }

        // generate Mars cipher, in CBC mode and NoPadding
        try {
            cp = Cipher.getInstance("Mars/CBC/NoPadding", "IBMJCE");
        } catch (NoSuchAlgorithmException ex) {
            System.out.println("Mars cipher not found in IBMJCE provider.");
            return;
        } catch (NoSuchProviderException ex) {
            // Should not happen.
            System.out.println("Provider IBMJCE not found.");
            return;
        } catch (NoSuchPaddingException ex) {
            // Should not happen.
            System.out.println("Invalid padding.");
            return;
        }

        // Encrypt the plain text
        try {
            cp.init(Cipher.ENCRYPT_MODE, MarsKey);
            cipherText = cp.doFinal(plainText);
            params = cp.getParameters();
        } catch (Exception ex) {
            System.out.println("Error encrypting: " + ex.getMessage());
            return;
        }

        // Decrypt the cipher text
        try {
            cp.init(Cipher.DECRYPT_MODE, MarsKey, params);
            newPlainText = cp.doFinal(cipherText);
        } catch (Exception ex) {
            System.out.println("Error decrypting: " + ex.getMessage());
            return;
        }

        // Verify the result.
        if (Arrays.equals(plainText, newPlainText)) {
            System.out.println("Encryption and decryption successful.");
        } else {
            System.out.println("Encryption and decryption not successful.");
        }
    }
}


