/*
 * %Z%%M%    %I%  %W%  %G%  %U%
 * ===========================================================================
 * IBM Confidential
 * OCO Source Materials
 * Licensed Materials - Property of IBM
 * IBM Security Software Development Kit, Java (tm) Technology Edition
 *
 * (C) Copyright IBM Corp. 1999, 2000 All Rights Reserved.
 *
 * The source code for this program is not published or otherwise divested of
 * its trade secrets, irrespective of what has been deposited with the U.S.
 * Copyright Office.
 * ===========================================================================
 */
 
//
// SAMPLE IBMJCE CODE
//

/*
 * Usage: This sample code demonstrates how to generate an RSA key 
 *        pair, and how to do RSA encryption and decryption.
 *
 
 * Requirement: Usage of this sample file requires the installation of
 *        the IBMJCE provider on the running machine and the CLASSPATH
 *        points to the classes.
 *
 *
 * Running the program by typing the following in a command line:
 *
 *       java SampleRSACrypto 
 *
 */


import java.io.*;
import java.util.Arrays;
import java.security.Security;
import java.security.Provider;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import javax.crypto.Cipher;
import javax.crypto.NoSuchPaddingException;

public class SampleRSACrypto {
    static final byte[] plainText = "Secret Message".getBytes();

    public static void main(String argv[]) {
        KeyPairGenerator rsaKeyPairGen;
        KeyPair rsaKeyPair;
        PublicKey rsaPub;
        PrivateKey rsaPriv;
        Cipher cp = null;
        byte[] cipherText = null, newPlainText = null;

        // Check if provider IBMJCE is installed.
        if (Security.getProvider("IBMJCE") == null) {
            // IBMJCE is not installed, install it.
            try {
                Security.addProvider
                ((Provider)Class.forName("IBMJCE").newInstance());
            }
            catch (Exception ex) {
                System.out.println("Cannot install provider: " + ex.getMessage());
                return;
            }
        }

        // Generate an RSA key pair
        try {
            rsaKeyPairGen = KeyPairGenerator.getInstance("RSA", "IBMJCE");
        } catch (NoSuchAlgorithmException ex) {
            System.out.println("RSA KeyPair Generator not found in IBMJCE provider.");
            return;
        } catch (NoSuchProviderException ex) {
            // Should not happen.
            System.out.println("Provider IBMJCE not found.");
            return;
        }

        // Use modulus length = 512
        rsaKeyPairGen.initialize(512);
        rsaKeyPair = rsaKeyPairGen.generateKeyPair();
        if (rsaKeyPair == null) {
            System.out.println("Key generation failure. KeyPair was not generated.");
            return;
        }

        // get RSA public key.
        rsaPub = rsaKeyPair.getPublic();

        // get RSA private key.
        rsaPriv = rsaKeyPair.getPrivate();

        // generate RSA cipher
        try {
            cp = Cipher.getInstance("RSA", "IBMJCE");
        } catch (NoSuchAlgorithmException ex) {
            System.out.println("RSA cipher not found in IBMJCE provider.");
            return;
        } catch (NoSuchProviderException ex) {
            // Should not happen.
            System.out.println("Provider IBMJCE not found.");
            return;
        } catch (NoSuchPaddingException ex) {
            // Should not happen.
            System.out.println("Invalid padding.");
            return;
        }

        // Encrypt the plain text
        try {
            cp.init(Cipher.ENCRYPT_MODE, rsaPub);
            cipherText = cp.doFinal(plainText);
        } catch (Exception ex) {
            System.out.println("Error encrypting: " + ex.getMessage());
            return;
        }

        // Decrypt the cipher text
        try {
            cp.init(Cipher.DECRYPT_MODE, rsaPriv);
            newPlainText = cp.doFinal(cipherText);
        } catch (Exception ex) {
            System.out.println("Error decrypting: " + ex.getMessage());
            return;
        }

        // Verify the result.
        if (Arrays.equals(plainText, newPlainText)) {
            System.out.println("Encryption and decryption successful.");
        } else {
            System.out.println("Encryption and decryption not successful.");
        }
    }
}


