/*
 * %Z%%M%    %I%  %W%  %G%  %U%
 * ===========================================================================
 * IBM Confidential
 * OCO Source Materials
 * Licensed Materials - Property of IBM
 * IBM Security Software Development Kit, Java (tm) Technology Edition
 *
 * (C) Copyright IBM Corp. 1999, 2000 All Rights Reserved.
 *
 * The source code for this program is not published or otherwise divested of
 * its trade secrets, irrespective of what has been deposited with the U.S.
 * Copyright Office.
 * ===========================================================================
 */
 
//
//     SAMPLE IBMJCE CODE
//

/*
 * Usage: This sample code demonstrate how to verify X509 Certificates.
 *
 
 * Requirement: Usage of this sample file requires the installation of
 *        the IBMJCE provider on the running machine and the CLASSPATH
 *        points to the classes.
 *
 *
 * Running the program by typing the following in a command line:
 *
 *       java SampleX509Verification CERTIFICATENUMBER
 *
 *       CERTIFICATENUMBER the certificate you want to verify (0-5)
 *       passing no argument will verify all certificates
 */

import java.security.interfaces.*;
import java.security.cert.*;
import java.security.cert.X509Certificate;
import java.math.*;
import java.io.*;

public class SampleX509Verification  {

	static final String[] CERTIFICATES_ENCODED_X509 = new String[6];

	static {

		CERTIFICATES_ENCODED_X509[0] =
	// taken from http://www.disa.mil/infosec/certs.txt
    "-----BEGIN CERTIFICATE-----\n" +
    "MIICZzCCAdCgAwIBAgIBGzANBgkqhkiG9w0BAQUFADBhMQswCQYDVQQGEwJVUzEY\n" +
    "MBYGA1UEChMPVS5TLiBHb3Zlcm5tZW50MQwwCgYDVQQLEwNEb0QxDDAKBgNVBAsT\n" +
    "A1BLSTEcMBoGA1UEAxMTRG9EIFBLSSBNZWQgUm9vdCBDQTAeFw05ODA4MDMyMjAy\n" +
    "MjlaFw0wODA4MDQyMjAyMjlaMGExCzAJBgNVBAYTAlVTMRgwFgYDVQQKEw9VLlMu\n" +
    "IEdvdmVybm1lbnQxDDAKBgNVBAsTA0RvRDEMMAoGA1UECxMDUEtJMRwwGgYDVQQD\n" +
    "ExNEb0QgUEtJIE1lZCBSb290IENBMIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKB\n" +
    "gQDbrM/J9FrJSX+zxFUbsI9Vw5QbguVBIa95rwW/0M8+sM0r5gd+DY6iubm6wnXk\n" +
    "CSvbfQlFEDSKr4WYeeGp+d9WlDnQdtDFLdA45tCi5SHjnW+hGAmZnld0rz6wQekF\n" +
    "5xQaa5A6wjhMlLOjbh27zyscrorMJ1O5FBOWnEHcRv6xqQIDAQABoy8wLTAdBgNV\n" +
    "HQ4EFgQUVrmYR6m9701cHQ3r5kXyG7zsCN0wDAYDVR0TBAUwAwEB/zANBgkqhkiG\n" +
    "9w0BAQUFAAOBgQDVX1Y0YqC7vekeZjVxtyuC8Mnxbrz6D109AX07LEIRzNYzwZ0w\n" +
    "MTImSp9sEzWW+3FueBIU7AxGys2O7X0qmN3zgszPfSiocBuQuXIYQctJhKjF5KVc\n" +
    "VGQRYYlt+myhl2vy6yPzEVCjiKwMEb1Spu0irCf+lFW2hsdjvmSQMtZvOw==\n" +
    "-----END CERTIFICATE-----\n";

		CERTIFICATES_ENCODED_X509[1] =
	// taken from http://www.aero.und.edu/certificate.html
    "-----BEGIN CERTIFICATE-----\n" +
    "MIICiTCCAfICAQAwDQYJKoZIhvcNAQEEBQAwgYwxCzAJBgNVBAYTAlVTMRUwEwYD\n" +
    "VQQIEwxOb3J0aCBEYWtvdGExFDASBgNVBAcTC0dyYW5kIEZvcmtzMRYwFAYDVQQK\n" +
    "Ew1VTkQgQWVyb3NwYWNlMRgwFgYDVQQDFA9yb290QGNzLnVuZC5lZHUxHjAcBgkq\n" +
    "hkiG9w0BCQEWD3Jvb3RAY3MudW5kLmVkdTAeFw05OTAzMDIyMDU4NDRaFw0wOTAy\n" +
    "MjcyMDU4NDRaMIGMMQswCQYDVQQGEwJVUzEVMBMGA1UECBMMTm9ydGggRGFrb3Rh\n" +
    "MRQwEgYDVQQHEwtHcmFuZCBGb3JrczEWMBQGA1UEChMNVU5EIEFlcm9zcGFjZTEY\n" +
    "MBYGA1UEAxQPcm9vdEBjcy51bmQuZWR1MR4wHAYJKoZIhvcNAQkBFg9yb290QGNz\n" +
    "LnVuZC5lZHUwgZ8wDQYJKoZIhvcNAQEBBQADgY0AMIGJAoGBALmlQJg5Nl2VsQZ1\n" +
    "/B8fW4YDdk09SlVc7vyWcbcmbWRBJee0jcH448XdRP/m/MXIRsVKyhLA5f01+VPy\n" +
    "E2aDkx2WiU4DpqbMbGGAytuXBNudJQmBXjWEFiAGe7dYgDNGKK7Yo1k49Q6qGg9q\n" +
    "5did3+ppsyfzbeaiDCH0LO5gegNvAgMBAAEwDQYJKoZIhvcNAQEEBQADgYEAJnok\n" +
    "1gvj4KC9HeUX/R4Q/f5tbJ2jLeQATIHtUx9QSKSq7IsdY0zz9EnKOsc9pr8JfBTL\n" +
    "cAwrxqvl5QuoCFVR2tQq8DtBQY8vp7bEF2CZVoxZJXMIKKiD/Hjb0oypbq5wF0SY\n" +
    "xN5DUfG5sShi+vPIAwE62tZ1P1I1N8DQpDYiXkw=\n" +
    "-----END CERTIFICATE-----\n";


		CERTIFICATES_ENCODED_X509[2] =
	// taken from http://thawte.com/support/crypto/certs.html
    "-----BEGIN CERTIFICATE-----\n" +
    "MIICWDCCAgICAQAwDQYJKoZIhvcNAQEEBQAwgbYxCzAJBgNVBAYTAlpBMRUwEwYD\n" +
    "VQQIEwxXZXN0ZXJuIENhcGUxEjAQBgNVBAcTCUNhcGUgVG93bjEdMBsGA1UEChMU\n" +
    "VGhhd3RlIENvbnN1bHRpbmcgY2MxHzAdBgNVBAsTFkNlcnRpZmljYXRpb24gU2Vy\n" +
    "dmljZXMxFzAVBgNVBAMTDnd3dy50aGF3dGUuY29tMSMwIQYJKoZIhvcNAQkBFhR3\n" +
    "ZWJtYXN0ZXJAdGhhd3RlLmNvbTAeFw05NjExMTQxNzE1MjVaFw05NjEyMTQxNzE1\n" +
    "MjVaMIG2MQswCQYDVQQGEwJaQTEVMBMGA1UECBMMV2VzdGVybiBDYXBlMRIwEAYD\n" +
    "VQQHEwlDYXBlIFRvd24xHTAbBgNVBAoTFFRoYXd0ZSBDb25zdWx0aW5nIGNjMR8w\n" +
    "HQYDVQQLExZDZXJ0aWZpY2F0aW9uIFNlcnZpY2VzMRcwFQYDVQQDEw53d3cudGhh\n" +
    "d3RlLmNvbTEjMCEGCSqGSIb3DQEJARYUd2VibWFzdGVyQHRoYXd0ZS5jb20wXDAN\n" +
    "BgkqhkiG9w0BAQEFAANLADBIAkEAmpIl7aR3aSPUUwUrHzpVMrsm3gpI2PzIwMh3\n" +
    "9l1h/RszI0/0qC2WRMlfwm5FapohoyjTJ6ZyGUUenICllKyKZwIDAQABMA0GCSqG\n" +
    "SIb3DQEBBAUAA0EAfI57WLkOKEyQqyCDYZ6reCukVDmAe7nZSbOyKv6KUvTCiQ5c\n" +
    "e5L4y3c/ViKdlou5BcQYAbxA7rwO/vz4m51w4w==\n" +
    "-----END CERTIFICATE-----\n";


		CERTIFICATES_ENCODED_X509[3] =
	// taken from http://www.aero.ufl.edu/~rll/pubkey.html
    "-----BEGIN CERTIFICATE-----\n" +
    "MIIDGjCCAtgCBDaRW4swCwYHKoZIzjgEAwUAMHMxCzAJBgNVBAYTAlVTMRAwDgYDVQQIEwdGbG9y\n" +
    "aWRhMRQwEgYDVQQHEwtHYWluZXN2aWxsZTEQMA4GA1UEChMHVW5rbm93bjEQMA4GA1UECxMHVW5r\n" +
    "bm93bjEYMBYGA1UEAxMPUm9iZXJ0IEx5YmFyZ2VyMB4XDTk5MDEwNTAwMjMzOVoXDTk5MDQwNTAw\n" +
    "MjMzOVowczELMAkGA1UEBhMCVVMxEDAOBgNVBAgTB0Zsb3JpZGExFDASBgNVBAcTC0dhaW5lc3Zp\n" +
    "bGxlMRAwDgYDVQQKEwdVbmtub3duMRAwDgYDVQQLEwdVbmtub3duMRgwFgYDVQQDEw9Sb2JlcnQg\n" +
    "THliYXJnZXIwggG3MIIBLAYHKoZIzjgEATCCAR8CgYEA/X9TgR11EilS30qcLuzk5/YRt1I870QA\n" +
    "wx4/gLZRJmlFXUAiUftZPY1Y+r/F9bow9subVWzXgTuAHTRv8mZgt2uZUKWkn5/oBHsQIsJPu6nX\n" +
    "/rfGG/g7V+fGqKYVDwT7g/bTxR7DAjVUE1oWkTL2dfOuK2HXKu/yIgMZndFIAccCFQCXYFCPFSML\n" +
    "zLKSuYKi64QL8Fgc9QKBgQD34aCF1ps93su8q1w2uFe5eZSvu/o66oL5V0wLPQeCZ1FZV4661FlP\n" +
    "5nEHEIGAtEkWcSPoTCgWE7fPCTKMyKbhPBZ6i1R8jSjgo64eK7OmdZFuo38L+iE1YvH7YnoBJDvM\n" +
    "pPG+qFGQiaiD3+Fa5Z8GkotmXoB7VSVkAUw7/s9JKgOBhAACgYBMhs/XcF0LAjbuhoAY4EOmxd4U\n" +
    "U0w4nSJQ2vKcgpyHU1Sv/tbUr3xEm6Yyx49j1eNp9jVwM1a6NYX8BO8fCSHIiUVvJVFlCcoO7Qb8\n" +
    "Px7drfbFAFt8mFE1mjYCuj21ePHhs1DlZKJwu2ElC6GaRwtBk3+oCMDAnLuySd0+fAohdDALBgcq\n" +
    "hkjOOAQDBQADLwAwLAIUddbqC3woMcABg/r1GPW9eVNStGwCFCBGySvdXK0i4aLVC4Ptbc3PQFjp\n" +
    "-----END CERTIFICATE-----\n";


		CERTIFICATES_ENCODED_X509[4] =
	// taken from http://www.aero.und.edu/ca/certificate.html
    "-----BEGIN CERTIFICATE-----\n" +
    "MIICyzCCAjQCAQAwDQYJKoZIhvcNAQEEBQAwga0xCzAJBgNVBAYTAlVTMREwDwYD\n" +
    "VQQIEwhOZXcgWW9yazEWMBQGA1UEBxMNTmV3IFlvcmsgQ2l0eTEcMBoGA1UEChMT\n" +
    "Q29sdW1iaWEgVW5pdmVyc2l0eTENMAsGA1UECxMEQWNJUzEfMB0GA1UEAxMWQ29s\n" +
    "dW1iaWEgVW5pdmVyc2l0eSBDQTElMCMGCSqGSIb3DQEJARYWY2VydC1hdXRoQGNv\n" +
    "bHVtYmlhLmVkdTAeFw05NzA0MjgxNDQxMDNaFw0wMDA0MjgxNDQxMDNaMIGtMQsw\n" +
    "CQYDVQQGEwJVUzERMA8GA1UECBMITmV3IFlvcmsxFjAUBgNVBAcTDU5ldyBZb3Jr\n" +
    "IENpdHkxHDAaBgNVBAoTE0NvbHVtYmlhIFVuaXZlcnNpdHkxDTALBgNVBAsTBEFj\n" +
    "SVMxHzAdBgNVBAMTFkNvbHVtYmlhIFVuaXZlcnNpdHkgQ0ExJTAjBgkqhkiG9w0B\n" +
    "CQEWFmNlcnQtYXV0aEBjb2x1bWJpYS5lZHUwgZ8wDQYJKoZIhvcNAQEBBQADgY0A\n" +
    "MIGJAoGBANiod6flzM72CbsK/3gzHzcdjpoozRDD/wgq31jEeDdfKY+ljAwxaZS9\n" +
    "mt7S1g7lL+55fx7FjfJxvJRXvS9UbDU46PDDyJloWYobg84bK5ZcV5UnIPZmGHW/\n" +
    "/xVDUtIGhc4T+Xm5p4F+4AcgewF2s4TbKWxfC98FJfepc31KjkGbAgMBAAEwDQYJ\n" +
    "KoZIhvcNAQEEBQADgYEAI/e6xC+osVM4eMkSUUWgihuocQlRL9ixTlGqW9fvNlI1\n" +
    "q58fELU5bcFko7d02S9Egac/9ckkt/sbHMv9zQhfnvpol8BN+LivGu+09IiOW4yq\n" +
    "c9xT58Pv9gwZ/Ei5VS+FXvzHIr91yWIlwLsnKfgYDrmQowG5FkHSG1ZotUdl7Oo=\n" +
    "-----END CERTIFICATE-----\n";



		CERTIFICATES_ENCODED_X509[5] =
	// taken from http://www.cusys.edu/~security/pki/repository/certificate_image.html
    "-----BEGIN CERTIFICATE-----\n" +
    "MIIC1TCCAj6gAwIBAgIBBDANBgkqhkiG9w0BAQQFADBZMQswCQYDVQQGEwJVUzEf\n" +
    "MB0GA1UEChMWVW5pdmVyc2l0eSBvZiBDb2xvcmFkbzEWMBQGA1UECxMNU3lzdGVt\n" +
    "IE9mZmljZTERMA8GA1UEAxMIVU1TIENBLTEwHhcNOTgwNTExMjEwMjU0WhcNMDgw\n" +
    "NTEwMjEwMjU0WjBZMQswCQYDVQQGEwJVUzEfMB0GA1UEChMWVW5pdmVyc2l0eSBv\n" +
    "ZiBDb2xvcmFkbzEWMBQGA1UECxMNU3lzdGVtIE9mZmljZTERMA8GA1UEAxMIVU1T\n" +
    "IENBLTEwgZ8wDQYJKoZIhvcNAQEBBQADgY0AMIGJAoGBALGf8Ny+kHlXqLTi3uIf\n" +
    "mkxxwDJd14sOg+hv85pxsqzCRJEgOx5YDUt05hJ7N0s4rJ/gNUcJaKR6ul+qLGbW\n" +
    "+Zb4S1YFbnKaO16zggvgckwpLGYRBbmee9+E47W8VEOZWrJXUkpJ/ZV8GAnesrvn\n" +
    "XTEKfm7bX9s6R7FQfDoHNnQfAgMBAAGjgawwgakwEQYJYIZIAYb4QgEBBAQDAgD2\n" +
    "MB8GA1UdIwQYMBaAFOqlID9Er6dI09n9Lvnby+FISi7oMFQGCWCGSAGG+EIBDQRH\n" +
    "FkVFeHRlbmRlZCBDQSBDZXJ0aWZpY2F0ZSBmcm9tIDI0IE1vbnRocyB0byAxMjAg\n" +
    "TW9udGhzLiAgRFRHID0gMDUxMTE5OTgwHQYDVR0OBBYEFOqlID9Er6dI09n9Lvnb\n" +
    "y+FISi7oMA0GCSqGSIb3DQEBBAUAA4GBAFNFo27JeeIgsMqS7Na//6gJQRilxwVS\n" +
    "Bfx6J43YX47EgNDLn4J7B9Tst+2bDZDAk1lZyu4y2WCLrnfg/e6B1KYBhCt/Srsc\n" +
    "r+WomFcw19k1jBtBaYxVwh/9N4ppZGdKILACciXbxfoLbbNgSDx5+KbE2c2m9is7\n" +
    "MIZgRexTvnJa\n" +
    "-----END CERTIFICATE-----\n";

    }

public static String toStringBlock(byte[] block) {
	if (block == null) return "null";
    return null;
}

public static void decodeX509(String certificate) {
	try {
		CertificateFactory cf = CertificateFactory.getInstance("X.509", "IBMJCE");

		ByteArrayInputStream bais = new ByteArrayInputStream(certificate.getBytes());
		while (bais.available() > 0) {
			System.out.println ("-----------------------------------------------------");
		    X509Certificate cert = (X509Certificate) cf.generateCertificate(bais);
		    System.out.println(cert.toString());
			System.out.println ("-----------------------------------------------------");
			System.out.println ("class: " + cert.getClass());
			System.out.println ("getNotAfter: " + cert.getNotAfter());
			System.out.println ("getNotBefore: " + cert.getNotBefore());
			System.out.println ("getSignature: " + cert.getSignature());
			System.out.println ("getVersion: " + cert.getVersion());
			System.out.println ("getIssuerDN: " + cert.getIssuerDN());
			System.out.println ("getSubjectDN: " + cert.getSubjectDN());
			System.out.println ("getSerialNumber: " + cert.getSerialNumber());
			System.out.println ("getPublicKey: " + cert.getPublicKey());
			System.out.println ("getSigAlgName: " + cert.getSigAlgName());
			System.out.println ("getSigAlgOID: " + cert.getSigAlgOID());
			System.out.println ("getSigAlgParams: " + cert.getSigAlgParams());
			System.out.println ("getKeyUsage: " + cert.getKeyUsage());
			System.out.println ("getIssuerUniqueID: " + cert.getIssuerUniqueID());
			System.out.println ("getSubjectUniqueID: " + cert.getSubjectUniqueID());
			System.out.println ("getBasicConstraints: " + cert.getBasicConstraints());
			System.out.println ("getKeyUsage: " + cert.getKeyUsage());
			try {
				cert.verify (cert.getPublicKey(), "IBMJCE");
				System.out.println ("signature verification OK");
			} catch (Exception e) {
				System.out.println ("signature verification -> " + e);
			}
		 }

	} catch (Throwable e) {
		e.printStackTrace();
	}
}
public static void main(String[] args) {

	java.security.Security.insertProviderAt ( new com.ibm.crypto.provider.IBMJCE() , 2);

	boolean oneCertOnly = false;
	int cert = 0;
	try {
		String certNumber = args[0];
		cert = Integer.parseInt (certNumber);
		oneCertOnly = true;
	} catch (Exception e) {}
	long start = System.currentTimeMillis();
	for (int i = 0; i < CERTIFICATES_ENCODED_X509.length; i++) {
		if (oneCertOnly && i != cert) continue;

		System.out.println ();
		System.out.println ();
		System.out.println ("=========================================================================");
		System.out.println (i + ")");
		System.out.println ("=========================================================================");
		decodeX509 (CERTIFICATES_ENCODED_X509[i]);
	}
	System.out.println ();
	System.out.println ();
	System.out.println ("Time to run: " + ((System.currentTimeMillis() - start) / 1000) + " seconds");
}


}
