/* 
 * %Z%%M%    %I%  %W%  %G%  %U%
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * IBM Security Software Development Kit, Java (tm) Technology Edition
 * 
 * (C) Copyright IBM Corp. 2000 All Rights Reserved.
 * 
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * ===========================================================================
 */

import java.security.NoSuchAlgorithmException; 
import java.io.UnsupportedEncodingException;
import com.ibm.security.pkcsutil.PKCSException;
import com.ibm.security.pkcs5.PKCS5;

/*
 * PKCS5EncryptDecrypt
 *
 * Sample program to demonstrate encryption and decryption operations using
 * the com.ibm.security.pkcs5.PKCS5 class.
 *
 * You can build this sample program class by issuing the following command:
 *
 *     javac PKCS5EncryptDecrypt.java
 *
 * Make sure that the ibmpkcs.jar file is in your CLASSPATH or extensions directory.
 *
 * To run this sample program, execute the following command:
 *
 *     java PKCS5EncryptDecrypt
 */

public class PKCS5EncryptDecrypt {

    public static void main(String[] args) {

        /*
         * The bytes to encrypt.
         */
        String dataString = "This is data to be encrypted.";
        byte[] dataBytes = null;
        try {
            dataBytes = dataString.getBytes("8859_1");
        } catch (UnsupportedEncodingException e) {
            System.out.println("Unsupported Encoding Expection: " + e);
            dataBytes = dataString.getBytes();
        }
        System.out.println("Original text: " + dataString);

        /*
         * The password.
         */
        byte[] password = { (byte)'p',      
            (byte)'0',     
            (byte)'1', 
            (byte)'s',
            (byte)'w',
            (byte)'5', 
            (byte)'8', 
            (byte)'d'}; 

        /*
         * The salt; this must be 8 bytes.
         */
        String saltString = "PKCS1234";
        byte[] salt = null;
        try {
            salt = saltString.getBytes("8859_1");
        } catch (UnsupportedEncodingException e) {
            System.out.println("Unsupported Encoding Expection: " + e);
            salt = saltString.getBytes();
        }

        /*
         * The iteration count; recommended to by 1000 or more.
         */
        int iterations = 1024;    

        /*
         * The message digest algorithm, key encryption algorithm, cipher
         * and version. 
         */
        String digestAlgorithm = "MD5";
        String keyAlgorithm = "MD5";
        String cipher = "TripleDES";
        int version = PKCS5.PKCS5_VERSION1;

        /*
         * Create a PKCS5 object.  
         */
        PKCS5 pkcs5 = new PKCS5(password, salt, iterations, digestAlgorithm,
                                cipher, keyAlgorithm, version);

        /*
         * Encrypt the plaintext bytes.
         */
        System.out.println("Encrypting the original text");
        byte[] encryptedBytes = null;
        try {
            encryptedBytes = pkcs5.encrypt(dataBytes);
        } catch (NoSuchAlgorithmException e) {
            System.out.println("Cipher " + cipher + " is not supported");
            System.exit(1);
        } catch (PKCSException e2) {
            System.out.println("Encryption error");
            e2.getRelatedException().printStackTrace();
            System.exit(1);
        }

        /* 
         * Create another PKCS5 object using the same arguments as the 
         * encrypting object and decrypt the encrypted data.
         */
        System.out.println("Decrypting the encrpyted text");
        PKCS5 anotherPkcs5 = new PKCS5(password, salt, iterations, 
                                       digestAlgorithm, cipher, keyAlgorithm, 
                                       version);
        byte[] decryptedBytes = null; 
        try {
            decryptedBytes = anotherPkcs5.decrypt(encryptedBytes);
        } catch (NoSuchAlgorithmException e) {
            System.out.println("Cipher " + cipher + " is not supported");
            System.exit(1);
        } catch (PKCSException e2) {
            System.out.println("Encryption error");
            e2.getRelatedException().printStackTrace();
            System.exit(1);
        }

        /*
         * Verify the results.
         */
        String decryptedString = null;
        try {
            decryptedString = new String(decryptedBytes, "8859_1");
        } catch (UnsupportedEncodingException e) {
            System.out.println("Unsupported Encoding Expection: " + e);
            decryptedString = new String(decryptedBytes);
        }
        System.out.println("Decrypted text: " + decryptedString);

        if (dataString.equals(decryptedString)) {
            System.out.println("SUCCESS: Original text matches decrypted text");
        } else {
            System.out.println("ERROR: Original text does NOT match decrypted text");
        }
    }
}
