/* 
 * %Z%%M%    %I%  %W%  %G%  %U%
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * IBM Security Software Development Kit, Java (tm) Technology Edition
 * 
 * (C) Copyright IBM Corp. 2000 All Rights Reserved.
 * 
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * ===========================================================================
 */

import java.security.NoSuchAlgorithmException;
import java.io.UnsupportedEncodingException;
import com.ibm.security.pkcsutil.PKCSException;
import com.ibm.security.pkcs5.PKCS5;

/*
 * PKCS5Mac
 *
 * Sample program to demonstrate how to generate and verify a message 
 * authentication code (MAC) using the com.ibm.security.pkcs5.PKCS5 class.
 *
 * You can build this sample program class by issuing the following command:
 *
 *     javac PKCS5Mac.java
 *
 * Make sure that the ibmpkcs.jar file is in your CLASSPATH or extensions directory.
 *
 * To run this sample program, execute the following command:
 *
 *     java PKCS5Mac
 */

public class PKCS5Mac {

    public static void main(String[] args) {

        /*
         * The bytes to authenticate.
         */
        String dataString = "Generating a MAC for this data.";
        byte[] dataBytes = null;
        try {
            dataBytes = dataString.getBytes("8859_1");
        } catch (UnsupportedEncodingException e) {
            System.out.println("Unsupported Encoding Expection: " + e);
            dataBytes = dataString.getBytes();
        }
        System.out.println("Data for the MAC: " + dataString);

        /*
         * The password.
         */
        byte[] password = { (byte)'p',      
                            (byte)'0',     
                            (byte)'1', 
                            (byte)'s',
                            (byte)'w',
                            (byte)'5', 
                            (byte)'8', 
                            (byte)'d' }; 

        /*
         * The salt; this must be 8 bytes.
         */
        String saltString = "PKCS1234";
        byte[] salt = null;
        try {
            salt = saltString.getBytes("8859_1");
        } catch (UnsupportedEncodingException e) {
            System.out.println("Unsupported Encoding Expection: " + e);
            salt = saltString.getBytes();
        }

        /*
         * The iteration count; recommended to by 1000 or more.
         */
        int iterations = 1024;    

        /*
         * The message digest algorithm, key encryption algorithm, cipher
         * and version. 
         */
         String digestAlgorithm = "MD5";
         String keyAlgorithm = "MD5";
         String cipher = "TripleDES";
         int version = PKCS5.PKCS5_VERSION1;

        /*
         * Create a PKCS5 object.  
         */
        PKCS5 pkcs5 = new PKCS5(password, salt, iterations, digestAlgorithm,
                                cipher, keyAlgorithm, version);

        /*
         * Generate the MAC. 
         */
        String hmacString = pkcs5.getHMACString();
        System.out.println("Generating a MAC on the data using " + hmacString);
        byte[] mac = null;
        try {
            mac = pkcs5.generateMAC(dataBytes);
        } catch (NoSuchAlgorithmException e) {
            System.out.println("The MAC algorithm " + hmacString + 
                               " is not supported"); 
            System.exit(1);
        } catch (PKCSException e) {
            System.out.println("The MAC could not be generated"); 
            e.getRelatedException().printStackTrace();
            System.exit(1);
        }

        /* 
         * Create another PKCS5 object using the same arguments as the 
         * object performing the MAC and verify the MAC. 
         */
        System.out.println("Verifying the MAC");
        PKCS5 anotherPkcs5 = new PKCS5(password, salt, iterations, 
                                       digestAlgorithm, cipher, keyAlgorithm, 
                                       version);

        try {
            anotherPkcs5.verifyMAC(mac, dataBytes);
            System.out.println("SUCCESS: The MAC is verified");
        } catch (PKCSException e) {
            System.out.println("ERROR: The MAC is NOT verified");
            e.getRelatedException().printStackTrace();
            System.exit(1);
        } catch (Exception e2) {
            System.out.println("Error verifying MAC: " + e2.toString());
            System.exit(1);
        }
    }
}
