/* 
 * %Z%%M%    %I%  %W%  %G%  %U%
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * IBM Security Software Development Kit, Java (tm) Technology Edition
 * 
 * (C) Copyright IBM Corp. 2000 All Rights Reserved.
 * 
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * ===========================================================================
 */

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Calendar;
import java.util.Date;
import com.ibm.security.pkcs9.SigningTime;
import com.ibm.security.pkcs9.UnstructuredName;
import com.ibm.security.pkcs9.PKCS9;
import com.ibm.security.pkcs9.PKCS9DerObject;

/*
 * PKCS9Pkcs9
 *
 * Sample program to demonstrate the creation of PKCS #9 attributes from a
 * DER-encoding using the com.ibm.security.pkcs9.PKCS9 class.
 *
 * You can build this sample program class by issuing the following command:
 *
 *     javac PKCS9Pkcs9.java
 *
 * Make sure that the ibmpkcs.jar file is in your CLASSPATH or extensions directory.
 *
 * To run this sample program, execute the following command:
 *
 *     java PKCS9Pkcs9
 */

public class PKCS9Pkcs9 {

    public static void main(String[] args) {

        /* 
         * Create a couple of attributes and DER-encode them.  The PKCS9
         * class is useful in instantiating attributes from byte streams. 
         */
        byte[][] attrs = new byte[2][];

        /*
         * A SigningTime attribute.
         */
        System.out.println("Creating a SigningTime attribute");
        Calendar cal = Calendar.getInstance();
        Date date = cal.getTime();
        SigningTime signingTime = new SigningTime(date);
        ByteArrayOutputStream attrBytes1 = new ByteArrayOutputStream();
        try {
            signingTime.encode(attrBytes1);
        } catch (IOException e) {
            System.out.println("Error encoding SigningTime attribute");
        }

        /*
         * An UnstructuredName attribute.
         */
        System.out.println("Creating an UnstructuredName attribute");
        String[] names = { "name1", "name2", "name3" };
        UnstructuredName unstructuredName = new UnstructuredName(names);
        ByteArrayOutputStream attrBytes2 = new ByteArrayOutputStream();
        try {
            unstructuredName.encode(attrBytes2);
        } catch (IOException e) {
            System.out.println("Error encoding SigningTime attribute");
        }

        attrs[0] = attrBytes1.toByteArray();
        attrs[1] = attrBytes2.toByteArray();

        /*
         * Use the PKCS9 class to instantiate the attributes in the generic
         * form of a PKCS9DerObject.
         */
        for (int i = 0; i < attrs.length; i++) {

            PKCS9DerObject attr = null; 

            /* Instantiate from the DER-encoding. */
            try {
                attr = PKCS9.getPKCS9Attribute(attrs[i]);
            } catch (IOException e) {
                System.out.println("Error generating attribute byte stream"); 
                continue;
            }

            /* Do some processing based on specific attribute type. */
            if (attr instanceof SigningTime) {
                System.out.println("SUCCESS: Got a SigningTime attribute"); 
            } else if (attr instanceof UnstructuredName) {
                System.out.println("SUCCESS: Got an UnstructuredName attribute"); 
            } else { 
                System.out.println("ERROR: Got an unexpected attribute"); 
            }
        }
    }
}
