    /* 
     * %Z%%M%    %I%  %W%  %G%  %U%
     * ===========================================================================
     * Licensed Materials - Property of IBM
     * IBM Security Software Development Kit, Java (tm) Technology Edition
     * 
     * (C) Copyright IBM Corp. 2000 All Rights Reserved.
     * 
     *  US Government Users Restricted Rights - Use, duplication or
     *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
     * ===========================================================================
     */
    
    import java.util.*;
    import java.io.*;
    import java.security.PrivateKey;
    import java.security.PublicKey;
    import java.security.Principal;
    import java.security.spec.PKCS8EncodedKeySpec;
    import java.security.KeyFactory;
    import java.security.cert.*;
    import com.ibm.security.pkcs7.*;
    import com.ibm.security.pkcs8.*;
    import com.ibm.security.pkcs10.*;
    import com.ibm.security.smime.*;
    import com.ibm.security.pkcsutil.*;
    import com.ibm.security.x509.*;
    import java.net.ConnectException;
    
    /*
     * SMIMESend
     *
     * Sample program to demonstrate how to create a SMIMESender object to send
     * S/MIME messages.  The program allows you to choose from the following actions:
     *
     * p7s - SignedData object with detached contents
     * p7m - SignedData object with contents
     * p7c - SignedData object with certificate only
     * e7m - EnvelopedData object wrapping SignedData with detached contents
     * ep7m - EnvelopedData object wrapping SignedData
     * e7o - EnvelopedData object
     * p10 - CertificationRequest object
     *
     * For general signed messages, use p7s.  For general encrypted messages,
     * use e7o, and for general signed and encrypted messages, use e7m.
     * For encrypted messages, specify a recipient certificate.
     *
     * This sample program uses two files found in the samples directory:
     *
     * sampleRSA.cer
     * This file contains the DER-encoding of an X.509 certificate.  This
     * certificate was generated by a certificate authority and contains an
     * RSA public key.
     *
     * sampleRSA.pri
     * This file contains the DER-encoding of an RSA private key.  This key is
     * the counterpart of the public key contained in the encoded certificate in
     * file sampleRSA.cer.  The key pair was generated by a KeyPairGenerator
     * instance.
     *
     * You can build this sample program class by issuing the following command:
     *
     *      javac SMIMESend.java
     *
     * Make sure that the ibmpkcs.jar file is in your CLASSPATH or extensions directory.
     *
     * For example, to run this sample program, execute the following command:
     *
     *     java samples.smime.SMIMESend -action=%ACTION% -sender=%SENDER_ADDRESS%
     *          -recipient=%RECIPIENT_ADDRESS% -mailhost=%MAILHOST% -cert=%SAMPLE_RSA_CER%
     *          -privatekey=%SAMPLE_RSA_PRI% -export
     *
     * where %ACTION% is the type of S/MIME message, such as p7s;
     * %SENDER_ADDRESS% is the to address of the message;
     * %RECIPIENT_ADDRESS% is the from address of the message;
     * %MAILHOST% is the mailhost domain name to send the message to;
     * %SAMPLE_RSA_CER% is a certificate file;
     * and %SAMPLE_RSA_PRI% is a private key file that is a companion to the public key
     * in the certificate.
     * The -export flag will optionally write the S/MIME message to a file.
     *
     * Additional parameters include -recipientcert for specifying the recipient's certificate
     * for an encrypted message, -password for an encrypted private key, -encrypt specifies
     * the level of encryption for a confidential message, and -subject overrides the default
     * subject line.
     */
    
    public class SMIMESend {
    
        public static void main(String args[]) {
    
            String dataString = "The quick brown fox jumps over the lazy dog.";
    
            String certfile = null;
            String privatekeyfile = null;
            Properties sysProp = System.getProperties();                          
            String fileSep = sysProp.getProperty("file.separator");           
            String defaultcertfile = ".." + fileSep + "sampleRSA.cer";
            String defaultprivatekeyfile = ".." + fileSep + "sampleRSA.pri";
            

            String password = null;
            String recipientcertfile = null;
    
            String signatureAlgorithm = "SHA1withRSA";
            String digestAlgorithm = "SHA1";
    
            String mailhost = null;
            String senderAddress = null;
            String recipientAddress = null;
            String defaultsubject = "This is a Test S/MIME Message";
            String subject = null;
            String action = "p7s";
            boolean bexport = false;
    
            String encrypt="medium";
            int encryptStrength = SMIME.MEDIUM_ENCRYPTION;
    
            String smimefilename = "smime.mail";
            String pkcsfilename = "smime";
    
            System.out.println(
                "Test program to create and mail a S/MIME SignedData, EnvelopedData,"
                + " or CertificationRequest object.");
            System.out.println(
                "Usage:  SMIMESend -sender=senderAddress -recipient=recipientAddress\r\n"
                + "\t-mailhost=mailhost [-subject=noteSubject]\r\n"
                + "\t[-action={p7s | p7m | p7c | e7m | e7o | p10 | ep7m}]\r\n"
                + "\t[-export] [-encrypt={weak | medium | strong}]\r\n"
                + "\t[-recipientcert=recipientCertFilename]\r\n"
                + "\t[-cert=senderCertFilename -privatekey=privateKeyFilename\r\n"
                + "\t-password=privateKeyPassword] |\r\n");
    
            try {
                for (int i=0;i<args.length;i++) {
                    String param = args[i].toLowerCase();
                    boolean bfound = false;
    
                    if (param.startsWith("-mailhost=")) {
                        mailhost = args[i].substring(10);
                        bfound = true;
                    }
                    if (param.startsWith("-sender=")) {
                        senderAddress = args[i].substring(8);
                        bfound = true;
                    }
                    if (param.startsWith("-recipient=")) {
                        recipientAddress = args[i].substring(11);
                        bfound = true;
                    }
                    if (param.startsWith("-subject=")) {
                        subject = args[i].substring(9);
                        bfound = true;
                    }
                    if (param.startsWith("-action=")) {
                        action = param.substring(8);
                        bfound = true;
                    }
                    if (param.startsWith("-export")) {
                        bexport = true;
                        bfound = true;
                    }
                    if (param.startsWith("-encrypt=")) {
                        encrypt = args[i].substring(9);
                        bfound = true;
                    }
                    if (param.startsWith("-recipientcert=")) {
                        recipientcertfile = args[i].substring(15);
                        bfound = true;
                    }
                    if (param.startsWith("-cert=")) {
                        certfile = args[i].substring(6);
                        bfound = true;
                    }
                    if (param.startsWith("-privatekey=")) {
                        privatekeyfile = args[i].substring(12);
                        bfound = true;
                    }
                    if (param.startsWith("-password=")) {
                        password = args[i].substring(10);
                        bfound = true;
                    }
    
                    if (!bfound) {
                        System.out.println("ERROR:  Unknown parameter " + args[i] + ".");
                        System.exit(1);
                    }
                }
    
                String[] validactions = {"p7c", "p7s", "p7m", "e7m", "e7o", "p10", "ep7m"};
                boolean baction = false;
    
                for (int i=0;i<validactions.length;i++) {
                    if (action.equals(validactions[i])) {
                        baction = true;
                    }
                }
    
                if (!baction) {
                    System.out.println("ERROR:  action must be p7s, p7m, p7c, e7m, e7o, p10, or ep7m.");
                    System.exit(1);
                }
    
                if ((!encrypt.equalsIgnoreCase("weak")) && ((!encrypt.equalsIgnoreCase("medium"))
                    && ((!encrypt.equalsIgnoreCase("strong"))))) {
                    System.out.println("ERROR:  encrypt must be weak, medium, or strong.");
                    System.exit(1);
                }
    
                if (encrypt.equalsIgnoreCase("weak")) {
                    encryptStrength = SMIME.WEAK_ENCRYPTION;
                } else if (encrypt.equalsIgnoreCase("strong")) {
                    encryptStrength = SMIME.STRONG_ENCRYPTION;
                }
    
                if (recipientAddress == null) {
                    System.out.println("ERROR:  recipientAddress must be specified.");
                    System.exit(1);
                }
    
                if (senderAddress == null) {
                    System.out.println("ERROR:  senderAddress must be specified.");
                    System.exit(1);
                }
    
                if (mailhost == null) {
                    System.out.println("ERROR:  mailhost must be specified.");
                    System.exit(1);
                }
    
                if (subject == null) {
                    subject = defaultsubject + " action = " + action;
                }
    
                System.out.println("**** Begin Program Parameters ****");
    
                System.out.println("Action Parameter: " + action);
    
                System.out.println("Mailing Parameters:");
                System.out.println("\tMail host = " + mailhost);
                System.out.println("\tSender address = " + senderAddress);
                System.out.println("\tRecipient address = " + recipientAddress);
                System.out.println("\tSubject = " + subject);
    
                System.out.println("Encryption Parameters:");
                if ((!action.equals("e7m")) && (!action.equals("e7o"))) {
                    System.out.println("\t*** Ignored since action is not e7m or e7o.");
                }
                System.out.println("\tEncryption Strength = " + encrypt);
                System.out.println("\tEncryption AlgorithmId = "
                    + SMIME.getEncryptionAlgorithm(encryptStrength));
                System.out.println("\tEncryption KeySize = "
                    + SMIME.getEncryptionKeySize(encryptStrength));
    
                System.out.println("Import Parameters:");
    
                if ((certfile != null) && (privatekeyfile != null)) {
                    System.out.println("\tSender Certificate Filename = " + certfile);
                    System.out.println("\tPrivate Key Filename = " + privatekeyfile);
                    System.out.println("\tPrivate Key Password = " + password);
                } else {
                    System.out.println("Using the default certificate file: " + defaultcertfile);
                    certfile = defaultcertfile;
                    System.out.println("Using the default private key file: " + defaultprivatekeyfile);
                    privatekeyfile = defaultprivatekeyfile;
                }
    
                System.out.println("\tRecipient Certificate Filename = " + recipientcertfile);
    
                if (((action.equals("e7m")) || (action.equals("e7o")))
                    && (recipientcertfile == null)) {
                    System.out.println(
                        "\tWARNING:  The recipient's certificate will default to the sender's certificate.");
                }
    
                pkcsfilename += "." + action;
    
                if (bexport) {
                    System.out.println("Export Parameters:");
                    System.out.println("\tSMIME Filename = " + smimefilename);
                    System.out.println("\tContentInfo/CertificationRequest Filename = "
                        + pkcsfilename);
                } else {
                    System.out.println("No objects will be exported.");
                }
    
                System.out.println("**** End Program Parameters ****");
    
                // Read the certificates and private key DER-encoded files.
    
                System.out.println("Reading in certificate = " + certfile);
    
                X509Certificate cert = null;
                X509Certificate recipientcert = null;
    
                FileInputStream fis = new FileInputStream((new File(certfile)).getAbsolutePath());
                CertificateFactory cf = CertificateFactory.getInstance("X509");
                cert = (X509Certificate) cf.generateCertificate(fis);
                fis.close();
    
                PublicKey publicKey = cert.getPublicKey();
    
                if (recipientcertfile != null) {
                    System.out.println("Reading in recipient certificate = "
                        + recipientcertfile);
    
                    fis = new FileInputStream((new File(recipientcertfile)).getAbsolutePath());
                    cf = CertificateFactory.getInstance("X509");
                    recipientcert = (X509Certificate) cf.generateCertificate(fis);
                    fis.close();
                } else {
                    // Default the recipient cert to the sender cert.
                    recipientcert = cert;
                }
    
                byte[] enckey = null;
                if (password != null) {
                    System.out.println("Reading in encrypted private key file = " + privatekeyfile);
                    EncryptedPrivateKeyInfo epki = new EncryptedPrivateKeyInfo(
                        privatekeyfile, false);
                    enckey = epki.decrypt(password.toCharArray());
                } else {
                    System.out.println("Reading in private key file = " + privatekeyfile);
                    fis = new FileInputStream((new File(privatekeyfile)).getAbsolutePath());
                    enckey = new byte[fis.available()];
                    fis.read(enckey);
                    fis.close();
                }
    
                PKCS8EncodedKeySpec keyspec = new PKCS8EncodedKeySpec(enckey);
                KeyFactory keyFactory = KeyFactory.getInstance("RSA");
                PrivateKey privateKey = keyFactory.generatePrivate(keyspec);
    
                // Create the SMIME object.
    
                System.out.println("*** Creating the SMIMESender object.");
    
                SMIMESender smime = null;
                ContentInfo contentInfo = null;
                CertificationRequest cr = null;
    
                if (action.equals("p7c")) {
                    Certificate[] certs = new Certificate[1];
                    certs[0] = cert;
                    SignedData sd = new SignedData(certs, null);
                    contentInfo = new ContentInfo(sd);
                    smime = new SMIMESender(contentInfo);
                } else if (action.equals("p7s")) {
                    smime = new SMIMESender();
                    smime.setMessageWithHeader(dataString);
                } else if (action.equals("p7m")) {
                    smime = new SMIMESender();
                    smime.setMessageWithHeader(dataString);
                
                } else if (action.equals("e7o")) {
                    smime = new SMIMESender();
                    smime.setMessageWithHeader(dataString);
                } else if (action.equals("ep7m")) {
                    if (privateKey == null) {
                        System.out.println(
                            "ERROR:  PrivateKey must be specified to create the SignedData"
                            + " object that is enclosed in the EnvelopedData object.");
                        System.exit(1);
                    }
                    SMIMESender smime2 = new SMIMESender();
                    smime2.setMessageWithHeader(dataString);
                    byte[] smime2msg = smime2.sign(cert, signatureAlgorithm, privateKey);
                    ContentInfo smime2ci = new ContentInfo(PKCSOID.DATA_OID, smime2msg);
                    smime = new SMIMESender(smime2ci);
                } else if (action.equals("e7m")) {
                    if (privateKey == null) {
                        System.out.println(
                            "ERROR:  PrivateKey must be specified to create the SignedData"
                            + " object that is enclosed in the EnvelopedData object.");
                        System.exit(1);
                    }
    
                    // First create the multipart message to encrypt.
    
                    SMIMESender smime2 = new SMIMESender();
                    smime2.setMessageWithHeader(dataString);
                    byte[] smime2msg = smime2.signSignatureOnly(cert, signatureAlgorithm,
                        privateKey);
    
                    ContentInfo smime2ci = new ContentInfo(PKCSOID.DATA_OID, smime2msg);
    
                    smime = new SMIMESender(smime2ci);
                } else { // action = p10
                    // Public key was previously retrieved from the certificate.
    
                    Principal princ = cert.getSubjectDN();
                    X500Name xname = new X500Name(princ.getName());
    
                    CertificationRequestInfo cri = new CertificationRequestInfo(xname,
                        publicKey, null);
                    cr = new CertificationRequest(cri, privateKey, digestAlgorithm);
    
                    smime = new SMIMESender(cr);
                }
    
                byte[] smimemsgbytes = null;
    
                System.out.println("\r\nSMIME Object = " + smime);
    
                if (action.equals("p7s")) {
    
                    System.out.println(
                        "*** Signing the SMIMESender object with multipart/signed.");
    
                    smimemsgbytes = smime.signSignatureOnly(cert, signatureAlgorithm,
                         privateKey);
    
                    System.out.println("\r\nSMIME Object after signature only signing = "
                        + smime);
                } else if (action.equals("p7m")) {
    
                    System.out.println("*** Signing the SMIMESender object.");
    
                    smimemsgbytes = smime.sign(cert, signatureAlgorithm, privateKey);
    
                    System.out.println("\r\nSMIME Object after signing = " + smime);
                } else if ((action.equals("e7m")) || ((action.equals("e7o")) 
                        || (action.equals("ep7m")))) {
    
                    System.out.println("*** Encrypting the SMIMESender object.");
    
                    // First set the recipient certificate.
    
                    Certificate[] recipientcerts = { recipientcert };
                    smime.setRecipientCertificate(recipientcerts);
    
                    // Encrypt the object.
    
                    smimemsgbytes = smime.encrypt(encryptStrength);
    
                    System.out.println("\r\nSMIME Object after encryption = " + smime);
                }
    
                // Export the S/MIME message.
    
                if (bexport) {
                    System.out.println("\r\n*** Exporting the S/MIME objects.");
    
                    contentInfo = smime.getContentInfo();
    
                    if (contentInfo != null) {
                        System.out.println("Writing the ContentInfo object to "
                            + pkcsfilename);
                        contentInfo.write(pkcsfilename, false);
                    } else {
                        System.out.println("Writing the CertificationRequest object to "
                            + pkcsfilename);
                        cr = smime.getCertificationRequest();
                        cr.write(pkcsfilename, false);
                    }
    
                    System.out.println("Writing the S/MIME message to " + smimefilename);
    
                    if (smimemsgbytes != null) {
                        smime.writeMailToFile(smimemsgbytes, smimefilename, senderAddress,
                            recipientAddress, subject);
                    } else {
                        smime.writeMailToFile(smimefilename, senderAddress,
                            recipientAddress, subject);
                    }
                }
    
                // Mail the SMIME contents.
    
                System.out.println("\r\n*** Mailing the SMIMESender object to "
                    + recipientAddress + ".\r\n\tSubject: " + subject);
    
                // If we have trouble connecting to the mail server, try up to four times.
    
                int btry = 0;
                boolean bdone = false;
    
                smime.setMailhost(mailhost);
    
                while (!bdone) {
                    try {
                        if (smimemsgbytes != null) {
                            smime.mail(smimemsgbytes, senderAddress, recipientAddress,
                                subject);
                        } else {
                            smime.mail(senderAddress, recipientAddress, subject);
                        }
                    } catch (ConnectException e) {
                        if (btry++ < 4) {
                            e.printStackTrace();
                            System.out.println("WARNING:  Trying to mail the message again.");
                            System.out.println("*** Retry #" + btry);
                            continue;
                        }
                        System.out.println("ERROR:  Could not mail the message after "
                            + btry + " tries.");
                    }
                    bdone = true;
                }
    
                System.out.println("EXIT:  SMIMESend from " + senderAddress + " to "
                    + recipientAddress + "\r\n\tsubject = " + subject);
    
            } catch (PKCSException e2) {
                System.out.println("\r\nERROR:  PKCSException: " + e2);
                e2.printStackTrace();
                Exception e3 = e2.getRelatedException();
                if (e3 != null) {
                    System.out.println("\r\nOriginal PKCSException stack:");
                    e3.printStackTrace();
                } else {
                    System.out.println("\r\nNo Original PKCSException stack available");
                }
            } catch (Exception e) {
                System.out.println("\r\nERROR:  Exception");
                e.printStackTrace();
            }
        }
    
    }
